/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright (c) Nuvoton Technology Corp. All rights reserved.                                             */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  This demo is used for testing M051 serials MCU Power down and Idlw mode, P20 
  interrupt is used to wakeup M051 from Power down or Idle mode, after 
  download the program, please power off and then pown on the chip, then test 
  can be start.
  Run the demo, press key 1 or key 2 for selection of Idle or Power down mode, 
  the way to wake up processor from Idle or Power down mode is generate a 
  falling edge at P2.0 pin. 
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP           0
#define CLOCK_EN              0x1F
#define PLL_Engine_Enable     0 
#define PLL_SEL               0x00080000 
#define CLOCK_SEL             0x0

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "UART.h"
#include "Retarget.h"

#define __WFI				  __wfi

static char Select_PD_IDLE_Mode = 0;

void P2_Init(uint32_t pin_num);

void delay(void);

int main(void)
{
	/* Unlock registers had been proctected */
	Un_Lock_Reg();	
			
	/* Enable external 12M Crystal */
	PWRCON |= XTL12M_EN;
	/* Wait until 12M clock is stable. */
	while((CLKSTATUS & XTL12M_STB) == 0); 

	/* HCLK clock source select 12M external Crystal */
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	

	/* Lock procteted registers */
	Lock_Reg();

	/* Initialize UART */
	UART_Init();

	do
	{
		printf("\n\n");
	    printf("Please select power-save mode...\n");
	    printf("+-----------------------------------------------------------+\n");
	    printf("| Idle mode                            	              - [1] |\n");
	    printf("| Power down mode                            	      - [2] |\n");
	    printf("+-----------------------------------------------------------+\n");
	    printf("| Quit                                        	    - [ESC] |\n");
	    printf("+-----------------------------------------------------------+\n");
	    printf("Select key : ");
		Select_PD_IDLE_Mode = getchar();
		printf("%c\n", Select_PD_IDLE_Mode); 

		/* Initialize P2.0 */
		P2_Init(0);

		switch(Select_PD_IDLE_Mode)
		{
			case '1':
				printf("Enter idle mode...\n");
				delay();
		
				Un_Lock_Reg();
				PWRCON &= ~PD_WAIT_CPU;
				PWRCON &= ~PWR_DOWN_EN;
				Lock_Reg();
				__WFI();
		
				delay();
				printf("Exit from idle mode...\n");
				break;
	
			case '2':
				printf("Enter power down mode...\n");
				delay();
		
				/* Enter power down mode */
				Un_Lock_Reg();
				/* Indicate waking from sleep might take longer. */
				SCR |= SLEEPDEEP;
				/* Disable interrupt occur when Power down mode wakeup */
				PWRCON &= ~PD_WU_IE;
				/* Power down control */
				PWRCON &= ~PD_WAIT_CPU;
				PWRCON |= PWR_DOWN_EN;
				Lock_Reg();
			
				delay();
				printf("WakeUp from power down mode...\n\n");
				break;
	
			default:
				break;
		}
	}while(Select_PD_IDLE_Mode != 27);
}

/*-----------------------------------------------------------------------------
  Function:		P2_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	pin_num: value is 0~7, corresponding to P20~P27.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize P2.x                                    
 *-----------------------------------------------------------------------------*/
void P2_Init(uint32_t pin_num)
{
	P2_IEN = 0;
	/* Configure P2[pin_num] as P2[pin_num] */
	P2_MFP &= ~((1 << pin_num) | (1 << (pin_num + 8)));
	
	/* Reset GPIO */
	IPRSTC2 |= GPIO_RST;
	IPRSTC2 &= ~GPIO_RST;

	/* Configure P2[pin_num] as input mode */
	P2_PMD &= (~(3<<(pin_num<<1)));	

	/* Configure P2[pin_num] interrupt trigger mode as edge-trigger */
	P2_IMD &= ~(1<<pin_num);

	DBNCECON &= ~ICLK_ON;
	DBNCECON &= DBCLK_HCLK;
	DBNCECON |= 8;

	/* Enable P2[pin_num] De-bounce function */
	P2_DBEN |= (1<<pin_num);

	/* Enable P2[pin_num] interrupt and wakeup function, configure P2[pin_num] interrupt as falling edge-trigger */
	P2_IEN |= (1<<pin_num);
	P2_IEN &= ~(1<<(pin_num+16));
	NVIC_ISER |= GP234_INT;		
} 

/*-----------------------------------------------------------------------------
  Function:		GPIOP2P3P4_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	P234 ISR routine                                    
 *-----------------------------------------------------------------------------*/
void GPIOP2P3P4_IRQHandler(void)
{
	uint32_t u32sfr;

	u32sfr = P2_ISRC;
	if (u32sfr != 0)
	{
		/* Clear P2 interrupt flag */
		P2_ISRC = u32sfr;
	}	
}

void delay(void)
{
	uint32_t i;

	for (i=0; i<10000; i++);
}
