/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------------
  M051 serials RTC demo code

  The PWM channels 0~3 (P2.0~3) will output waveforms according to the 
  setting in configuration wizard.

  Nuvoton Technoledge Corp. PU20 KEHsu
  19, Apr, 2010
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//                  <q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//  </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
// <h> PWM Mode Select
//    <e5.4> PWM0 and 1 Complement Mode Enable
//          <o6.16..23> PWM0 dead zone interval <0-255>   
//    </e>
//    <e5.5> PWM2 and 3 Complement Mode Enable
//          <o6.24..31> PWM2 dead zone interval <0-255>   
//    </e>
// </h>
// <h> PWM Enalbe
//                  <q7.0> PWM0 enable    <q7.8> PWM1 enable
//                  <q7.16> PWM2 enable   <q7.24> PWM3 enable
// </h>
// <h> PWM Clock Setting
//    <h> PWM Clock Source Select
//       <o8.28..29> PWM0 and 1 Clock Source Select
//                  <0=> Crystal 12MHz    
//                  <2=> HCLK             <3=> Internal RC 22MHz
//       <o8.30..31> PWM2 and 3 CLock Source Select
//                  <0=> Crystal 12MHz    
//                  <2=> HCLK             <3=> Internal RC 22MHz
//    </h>
//    <h> PWM Clock Pre-scalar
//       <o9.0..7> PWM0 and 1 Clock Pre-scalar <2-256> <#-1>
//       <o9.8..15> PWM2 and 3 CLock Pre-scalar <2-256> <#-1>
//    </h>
//    <h> PWM Clock Divider
//       <o10.0..2> PWM0 Clock Divider
//                  <4=> 1/1 <0=> 1/2 <1=> 1/4 <2=> 1/8 <3=> 1/16
//       <o10.4..6> PWM1 Clock Divider
//                  <4=> 1/1 <0=> 1/2 <1=> 1/4 <2=> 1/8 <3=> 1/16
//       <o10.8..10> PWM2 Clock Divider
//                  <4=> 1/1 <0=> 1/2 <1=> 1/4 <2=> 1/8 <3=> 1/16
//       <o10.12..14> PWM3 Clock Divider
//                  <4=> 1/1 <0=> 1/2 <1=> 1/4 <2=> 1/8 <3=> 1/16
//    </h>
// </h>
//    <h> PWM Output Setting
//    <h> PWM Output Invert
//                  <q11.2> PWM0 output invert    <q11.10> PWM1 output invert (invalid if complement mode)
//                  <q11.18> PWM2 output invert   <q11.26> PWM3 output invert (invalid if complement mode)
//    </h>
//    <h> PWM Output Enalbe
//                  <q12.0> PWM0 output enable    <q12.1> PWM1 output enable
//                  <q12.2> PWM2 output enable    <q12.3> PWM3 output enable
//    </h>
// </h>
//
//-------- <<< end of configuration section >>> ------------------------------
*/

#include <stdint.h>
#include "common.h"
#include "Register_Bit.h"
#include "M051.h"

#define CLOCK_SETUP             1
#define CLOCK_EN                0xF
#define PLL_Engine_Enable       0 
#define PLL_SEL                 0x00080000 
#define CLOCK_SEL               0x0
#define COMPLEMENT_MODE         0x00000020
#define DEAD_ZONE_INTERVAL      0xC8FF0000
#define PWM_ENABLE              0x01010101    
#define PWM_CLOCK_SOURCE        0xF0000000 
#define PWM_PRESCALAE           0x0000C763
#define PWM_CLOCK_DIVIDER       0x00003210
#define PWM_OUTPUT_INVERT       0x00040000
#define PWM_OUTPUT_ENABLE       0x0000000F

/*-----------------------------------------------------------------------------
  Function:		PWMA_IRQHandler                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	WMA ISR routine                                    
 *-----------------------------------------------------------------------------*/
void PWMA_IRQHandler(void)
{
    PIIRA = PIIRA;                            //PWM flags clearing by writing "1"
}

main(void)
{
    Un_Lock_Reg();
    PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock

    P2_MFP |= ~(P20_AD8_PWM0 |
			   	P21_AD9_PWM1 |
			   	P22_AD10_PWM2|
			   	P23_AD11_PWM3);
	P2_MFP |= (PWM0 | PWM1 | PWM2 | PWM3);	  //Switch P2.0~3 to PWM0~3 output
   
    P2_PMD &= ~Px0_PMD;                       //Configure PWM0~3 output pins "push-pull" mode
	P2_PMD |= Px0_OUT;

	P2_PMD &= ~Px1_PMD;
	P2_PMD |= Px1_OUT;

	P2_PMD &= ~Px2_PMD;
	P2_PMD |= Px2_OUT;

	P2_PMD &= ~Px3_PMD;
	P2_PMD |= Px3_OUT;

    APBCLK |= PWM01_CLKEN|
			  PWM23_CLKEN;                                      //Enable PWM0,1 and PWM2,3 clock
	CLKSEL1 = PWM_CLOCK_SOURCE;           						//Select PWM0,1 and PWM2,3 clock source
    
	PPRA = PWM_PRESCALAE | DEAD_ZONE_INTERVAL;                  //Select PWM0,1 and pWM2,3 clock pre-scalar and PWM2,3 dead-zone scale
    CSRA = PWM_CLOCK_DIVIDER;                                   //Select PWM0~3 clock divider
    PCRA = 0x08080808 | PWM_OUTPUT_INVERT | COMPLEMENT_MODE;    //PWM under auto-reload (continuous) mode, select output channel invertion, set complement mode

    CNR0A = CNR1A = CNR2A = CNR3A = 0x1000;                     //PWM period
    CMR0A = CMR1A = 0x800;                                      //PWM0,1 duty 50%
    CMR2A = CMR3A = 0x400;                                      //PWM2,3 duty 25%

    PIERA  |= PWMIE3 |                                          //Enable PWM0~3 interrupt
   			  PWMIE2 |
			  PWMIE1 |
			  PWMIE0 ;	
    NVIC_ISER |= PWMA_INT;                                      //Enable NVIC PWM0~3 interrupt

    POEA = PWM_OUTPUT_ENABLE;                                   //Output enable
    PCRA |= PWM_ENABLE;                                         //PWM circuit enable and start to run

    while(1);
}
