/*---------------------------------------------------------------------------------------------------------*/
/*                                                                                                         */
/* Copyright(c) 2010 Nuvoton Technology Corp. All rights reserved.                                         */
/*                                                                                                         */
/*---------------------------------------------------------------------------------------------------------*/
 
/*-----------------------------------------------------------------------------
	Nuvoton Technoledge Corp creadted by PU20 

	The function of this demo is to access a block of ram space. First, write 
	some data into a specified block of ram space, then read the block, verify 
	the read out data is equal to has been written or not, if equal, the 
	operation is successful, otherwise, it is failed. Success or fail message 
	will be shown on Hyper-terminal.
 
    SRAM address: 0x2000_0000 ~ 0x2000_0FFF
 *-----------------------------------------------------------------------------*/

/*
//-------- <<< Use Configuration Wizard in Context Menu >>> ------------------
//
// <e0> Clock Configuration
//   <h> Enable Clock Source (PWRCON)
//                  <q1.0> Crystal 12MHz
//					<q1.3> Internal RC 10kHz
//                  <q1.2> Internal RC 22MHz
//	 </h>
// <e2> Enable PLL (PLLCON)
//    <o3.19> PLL Clock Source Selection
//                  <0=> Crystal 12MHz        <1=> Internal RC 22MHz
// </e2>
//    <o4.0..2> HCLK System Clock Source Selection (CLKSEL0)
//                  <0=> Crystal 12MHz        
//                  <2=> PLL Clock            <3=> Internal RC 10KHz
//                  <4=> Internal RC 22MHz
// </e0>
//-------- <<< end of configuration section >>> ------------------------------
*/

#define CLOCK_SETUP             0
#define CLOCK_EN                0xF
#define PLL_Engine_Enable       1 
#define PLL_SEL                 0x00080000 
#define CLOCK_SEL               0x2

#include <stdio.h>
#include <stdint.h>
#include "M051.h"
#include "Register_Bit.h"
#include "Common.h"
#include "UART.h"
#include "Macro_SystemClock.h"
#include "Macro_Timer.h"

#define SRAM_BUF ((volatile unsigned long *) (0x20000000+0x00000000))

void TMR0_Delay1ms(uint32_t ulCNT);
void Timer0_Init(void);
void UART_Init(void);
void Show_SRAM_Read_Error(void);
void SRAM_Read(uint32_t ulData);
void SRAM_Write(uint32_t ulData);

/*-----------------------------------------------------------------------------
  Function:		TMR0_Delay1ms                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulCNT:	delay time, unit is millisecond.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Timer0 used for a delay function.                                    
 *-----------------------------------------------------------------------------*/
void TMR0_Delay1ms(uint32_t ulCNT)
{
    TCMPR0 = 12000;         //Fosc=12MHz, so 12000000/12000=1000Hz=1ms
    setTMR0_CEN;            //Start timer0
    while (ulCNT != 0)
    {
        while ((TISR0&TMR_TIF) != TMR_TIF); //check TIF0
        TISR0 |= TMR_TIF;   //Clear TIF0
        ulCNT --;
    }
    clrTMR0_CEN;            //Stop timer0
}

/*-----------------------------------------------------------------------------
  Function:		Timer0_Init                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Initialize timer0, period mode.                                    
 *-----------------------------------------------------------------------------*/
void Timer0_Init(void)
{
    TMR0_Clock_EN;
	TMR0ClkSource_ex12MHz;
    TCSR0  = 0x00000000;    //Pre-Scaler
    
    setTMR0_PERIOD;
    setTMR0_IE;             //Timer0 interrupt enable
    setTMR0_CRST;           //Reset the timer/counter0, after set, this bit will be clear by H/W
}

/*-----------------------------------------------------------------------------
  Function:		Show_SRAM_Read_Error                                                                         
                                                                                                         
  Parameter:        																					   	
	        	None                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Print error messege when verify read out data is wrong.                                    
 *-----------------------------------------------------------------------------*/
void Show_SRAM_Read_Error(void)
{
    printf("SRAM_Read Error\n");
    while(1);
}

/*-----------------------------------------------------------------------------
  Function:		SRAM_Write                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulData:		Data to be written to specified ram space.                                    
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Write specified ram space.                                    
 *-----------------------------------------------------------------------------*/
void SRAM_Write(uint32_t ulData)
{
    uint16_t uiCount;

    for(uiCount=0;uiCount<0x100;uiCount++) //0x100*4=0x400=1K Bytes
    {
        SRAM_BUF[uiCount] = ulData;
        ulData = ~ulData;
    }
}

/*-----------------------------------------------------------------------------
  Function:		SRAM_Read                                                                         
                                                                                                         
  Parameter:        																					   	
	        	ulData:		Data to be compared with specified ram space.                                     
  Returns:                                                                                                
               	None                                                                                      
  Description:                                                                                            
               	Read specified ram space and verify read out data.                                    
 *-----------------------------------------------------------------------------*/
void SRAM_Read(uint32_t ulData)
{
    uint16_t uiCount;

    for(uiCount=0;uiCount<0x100;uiCount++) //0x100*4=0x400=1K Bytes
    {
        if(ulData != SRAM_BUF[uiCount])
            Show_SRAM_Read_Error();
        else
            ulData = ~ulData;
    }
}

main(void)
{
    Un_Lock_Reg();
    PWRCON |= XTL12M_EN;
	while((CLKSTATUS & XTL12M_STB) == 0); 		//Wait until 12M clock is stable.	
	
	CLKSEL0 = (CLKSEL0 & (~HCLK)) | HCLK_12M;	//Set external crystal as the system clock
    
    UART_Init();
	printf("Press Enter to start...\n");
	getchar();

    SRAM_Write(0x55AA55AA);
    SRAM_Read (0x55AA55AA);

    SRAM_Write(0x00FF00FF);
    SRAM_Read (0x00FF00FF);

    SRAM_Write(0x00000000);
    SRAM_Read (0x00000000);

    printf("SRAM test is successful\n");
    while(1);
}
